# DirArg V0.1
#
# A Zope product for passing arguments via URL
#
# Copyright (C) by n@work Internet Informationssysteme GmbH 2000
#
# DirArg is distributed under the GNU LESSER GENERAL PUBLIC LICENSE (LGPL)
# Version 2.1 (February 1999). See the "COPYING" file distributed with
# this software for more info.
#
# Changes
#	initial release
#
# Status
#	initial release
#
# Bugs
#	hmmm...
#

# Documentation string. I wonder what ever refers to it?
# (BaseRequest.py does. The traverse method checks for an
# existing __doc__ string. If it does not exists an error
# is raised.)

__doc__ = """DirArg Product V0.1

A Zope product for passing arguments via URL

Copyright (C) by n@work Internet Informationssysteme GmbH 2000

DirArg is distributed under the GNU LESSER GENERAL PUBLIC LICENSE (LGPL)
Version 2.1 (February 1999). See the "COPYING" file distributed with
this software for more info."""

# Version string. Usually updated automatically by CVS.

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

import OFS.Folder
import Acquisition
import AccessControl.Role
import ExtensionClass

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addDirArgForm = HTMLFile('dirargAdd', globals()) 

def write_debug(text):
    boring_output = open('/tmp/dirarg_output.txt','a')
    boring_output.write(text+'\n')
    boring_output.close()


def manage_addDirArg(self, id, title='', REQUEST=None):
    """Add a DirArg folder to a folder."""
    self._setObject(id, DirArg(id, title))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

"""Step #2: Provide the class for DirArg object."""

class DirArg(
    OFS.Folder.Folder,     # DirArg is a folder-like object
    OFS.ObjectManager.ObjectManager,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
        
    meta_type = 'DirArg Folder' 

    def __init__(self, id, title=''): 
        """initialise a new instance of DirArg"""
        self.id = id
        self.title = title

    """Here starts the magic. Zope traverses an URL step by step.
       You really should have a look inside the BaseRequest.py
       file. The traverse method is what this is all about.

       At one point traverse will check the current traversed object
       for an existing __bobo_traverse__ method. If so, the method
       is called. The return value of __bobo_traverse__ will be a
       subobject that will be called in the next round of traverse.

       __bobo_traverse__ is called with the REQUEST object and 'name'
       which is the name of the object to be traversed next.

       DirArg has two stages of __bobo_traverse__. The first stage
       is entered, when traverse is hitting a DirArg object for the
       first time.

       We will then check whether the next object to be traversed
       (stored in name) is available inside the DirArg Folder itself.
       In this case, just return this object.

       If no object is available, we switch to stage two. All directory
       information that is not available as a real object will be
       stored as variable information. This is done by a helper class
       'Traverse'. This class provides a second __bobo_traverse__ that
       will eat all names in the traverse process until there is no
       more object for zope to traverse or an object is available inside
       the DirArg Folder. Oh well... hope you can follow."""

    def __bobo_traverse__(self, REQUEST, name):
        #write_debug('myself: '+getattr(self,'id')+' calling name:'+name)

        # is 'name' available ?
        if name and hasattr(self,name):
            return getattr(self,name)

        elif not name:
            """this never should happen since Zope always will
               ask for an index_html. (see BaseRequest.py,
               look in traverse for "Set the default method" and
               a few lines below "request._hacked_path=1" """
            # write_debug('no name just returning index_html (hmmm...)')
            return self
        else:
            """No object with this name is available. So we
               enter stage two. The helper class Traverse is
               instantiated and the traverse request is handed
               over to __bobo_traverse__ of Traverse."""
            #write_debug('called Traverse with '+name)
            return Traverse(self,name).__bobo_traverse__(REQUEST, name)
    
class Traverse:
    """Helper class for traversing.
       Ok... why a helper class ? Just because :-) Well this all
       may actually work without this helper class. One reason for
       choosing a helper class is because of some temporary variables.
       I can't store it inside the DirArg Folder itself, since
       it would make them persistent. And I guess it wouldn't be much
       fun with more than one user making calls to this object.
       The helper class will go away after the whole request."""

    def __init__(self, caller, name=None):
        self._args={}         # dictonary that stores name/value pairs
        self._caller=caller   # reference back to our main object
        self._arg=None        # temporary argument name
        self._arg_list=[]     # list of all URL names
        
    def __bobo_traverse__(self, REQUEST, name):
        
        #write_debug('inside traverse: calling name:'+name)
        #write_debug('trns: '+str(REQUEST['TraversalRequestNameStack']))

        """saving names:
           (remember: this method will be called for every
           following traversed URL name until the URL is
           done or an object is available in the main DirArg
           object)
           name will hold the actual traversed URL name.

           first we will take a name as an argument name.
           The value of this argument is None. The name
           will be stored in _arg to remember it in the
           next run.

           The next name will be saved as the value of
           the last name.

           If an argument name will be used twice or more
           (ie http://www.blub.de/DirArgFolder/list/2/list/3)
           The value will be transformed into a list object."""

        self._arg_list.append(name)

        if self._arg is None:
            self._arg = name
            if not self._args.has_key(name):
                self._args[name]=None
        else:
            if self._args[self._arg] is None:
                self._args[self._arg]=name
            elif type(self._args[self._arg]) is not type([]):
                self._args[self._arg]=[self._args[self._arg],name]
            else:
                self._args[self._arg].append(name)
            self._arg=None

        """Now we have to decide whether to leave the helper
           class or to continue. In case of leaving all consumed
           data will be promoted into the REQUEST object for
           access inside your methods/objects."""
            
        if len(REQUEST['TraversalRequestNameStack']) == 0:
            """Oh yes... TraversalRequestNameStack. Cool long name.
               traverse inside BaseRequest.py stores the remaining
               objects there and we are using it for deciding when it
               is time to leave. In this case it is time to go,
               since the stack is empty. So we hand our data over
               to REQUEST and return our parent to the traverse
               machina"""
            #write_debug('traverse: trns is empty... returning')
            self.fill_request(REQUEST)
            return self._caller
        elif hasattr(self._caller,REQUEST['TraversalRequestNameStack'][-1]):
            """If the next object will be available inside our parent,
               we have to leave..."""
            #write_debug('traverse: next trns element is for caller ... returning')
            self.fill_request(REQUEST)
            return self._caller
        else:
            """We have saved 'name' and there is more to come...
               We return this object to traverse, so we will be
               called directly in the next round."""
            #write_debug('traverse: i was '+name+' ... leaving')
            return self

    def fill_request(self,REQUEST):
        REQUEST['DirArg_dict'] = self._args
        REQUEST['DirArg_list'] = self._arg_list
        for i in self._args.keys():
            REQUEST[i]=self._args[i]
        

    
